

close all;

clear;

addpath('D:\COMPUTATION\MATLAB\PennesBioHeatSolver');

% load SAR map
load SAR;
SAR=Value_of_SAR_0s*1000;  % the factor of 1000 comes from the scaling of the SEMCAD temp simulation

dx=x_axis(2)-x_axis(1);
dy=y_axis(2)-y_axis(1);
dz=z_axis(2)-z_axis(1);

[xxs yys zzs]=ndgrid(x_axis,y_axis,z_axis);

options.KeepAllTempMaps=0;
options.DisplayTemp=1;
options.BoundaryTemperature=0;
options.TimeStep=19.99;
options.UsePreconditioner=1;
options.LUPreconDropTol=0.05;
options.UniformThermalCondMap=0;

mask=SAR>0;
mask_rect=abs(xxs)<0.02 & abs(yys)<0.02 & abs(zzs)<0.02;

T0=zeros(size(mask));
rho=mask*1000;
C=mask*2000;
C(find(mask_rect>0))=4000;
K=mask*1;
K(find(mask_rect>0))=2;
Wb=zeros(size(mask));
Cb=0;
Tb=0;
Q=double(SAR.*rho);

to_disp1=[ squeeze(T0(:,:,end/2)) squeeze(T0(:,end/2,:)) squeeze(T0(end/2,:,:)) ];
to_disp2=[ squeeze(rho(:,:,end/2)) squeeze(rho(:,end/2,:)) squeeze(rho(end/2,:,:)) ];
to_disp3=[ squeeze(C(:,:,end/2)) squeeze(C(:,end/2,:)) squeeze(C(end/2,:,:)) ];
to_disp4=[ squeeze(K(:,:,end/2)) squeeze(K(:,end/2,:)) squeeze(K(end/2,:,:)) ];
to_disp5=[ squeeze(Wb(:,:,end/2)) squeeze(Wb(:,end/2,:)) squeeze(Wb(end/2,:,:)) ];
to_disp6=[ squeeze(Q(:,:,end/2)) squeeze(Q(:,end/2,:)) squeeze(Q(end/2,:,:)) ];

f1=figure;
imagesc(to_disp1); axis image; axis off; colormap(hot); colorbar;
title('T0 [degrees C]');

f2=figure;
imagesc(to_disp2); axis image; axis off; colormap(hot); colorbar;
title('Density [kg/m^3]');

f3=figure;
imagesc(to_disp3); axis image; axis off; colormap(hot); colorbar;
title('Specific heat (C) [J/(kg*degree)]');

f4=figure;
imagesc(to_disp4); axis image; axis off; colormap(hot); colorbar;
title('Thermal conductivity (K) [W/(m*degree)]');

f5=figure;
imagesc(to_disp5); axis image; axis off; colormap(hot); colorbar;
title('Perfusion (Wb) [kg/(s*m^3)]');

f6=figure;
imagesc(to_disp6); axis image; axis off; colormap(hot); colorbar;
title('Heat rate generation (Q) [W/m^3]');

% solve for temperature
[Ts times mask]=pbhs_solve(T0,1800,double([dx dy dz]),rho,C,K,Wb,Cb,Tb,Q,options);

% display results
T_pbhs=Ts;

load TEMP_SEMCAD;
T_SEMCAD=Value_of_T_t__1799p1s;

to_disp7=[ squeeze(T_pbhs(:,:,end/2)) squeeze(T_pbhs(:,end/2,:)) squeeze(T_pbhs(end/2,:,:)) ];
to_disp8=[ squeeze(T_SEMCAD(:,:,end/2)) squeeze(T_SEMCAD(:,end/2,:)) squeeze(T_SEMCAD(end/2,:,:)) ];

f7=figure;
imagesc(to_disp7); axis image; axis off; colormap(hot); colorbar;
title('PBHS temp. map [W/m^3]');

f8=figure;
imagesc(to_disp8); axis image; axis off; colormap(hot); colorbar;
title('SEMCAD temp. map [W/m^3]');




































