



clear;
close all;

addpath('F:\Computation\MATLAB\PennesBioHeatSolver');

mask = ones(64,64,64);
mask(1,:,:) = 0;
mask(end,:,:) = 0;

mask(:,1,:) = 0;
mask(:,end,:) = 0;

mask(:,:,1) = 0;
mask(:,:,end) = 0;

SAR = zeros(64,64,64);
SAR(32,32,32) = 1e6;  % W/kg

% SAR = smooth3d(SAR,7);

T0 = mask * 0;  % Celscius
DUR = 10 * 60;  % s
DX = 2e-3;  % pixel size (m)

RHO = mask * 1000;  % kg/m^3
C = mask * 3648;  % J/kg/K
K = mask * 0.5300;  % W/m/K
Wb = (mask*435) * 1e-6 / 60;  % mL/min/kg  ->  m^3/s/kg

rho_b=1000;
Wb = Wb .* rho_b .* RHO;  % m^3/kg/s -> kg/m^3/s

Cb = 3617;  % J/kg/K
Tb = 0;  % Celscius
Q = SAR .* RHO;  % W/m^3

options.KeepAllTempMaps = 1;
options.DisplayTemp = 1;
options.UsePreconditioner = 0;
options.LUPreconDropTol = 0.05;
options.BoundaryConditionType = 1;  % Dirichlet
options.BoundaryTemperature = 0;


% STEADY-STATE
T_SS_crit = pbhs_solve_SS( [DX DX DX] , RHO , C , K , Wb , Cb , Tb , Q , options );



% APPROACH TO EQUILIBRIUM
[ Ts_crit times ] = pbhs_solve( T0 , DUR , [DX DX DX] , RHO , C , K , Wb , Cb , Tb , Q , options );


f = figure;

MIP1 = squeeze( max(T_SS_crit,[],3) );
MIP2 = squeeze( max(Ts_crit(:,:,:,end),[],3) );

imagesc( [MIP1 MIP2] )  ; axis image off; colorbar;
title('Left: SS  //  Right: Approach to SS');


f = figure;

[VAL IND] = max(T_SS_crit(:));
TEMP = permute( Ts_crit,[4 1 2 3] );
y = TEMP(:,IND);

plot( times,y,'-or' ); hold on;
plot( times,ones(size(times))*VAL,'-k' );

xlabel('Time [sec.]'); ylabel('Temperature [K]');

legend('Approach to SS','SS');















































