


function L=pbhs_create_laplacian_operator(mask,DX,kmap,k_non_unif)
% function L=pbhs_create_laplacian_operator(mask,DX,kmap,k_non_unif)

[nx ny nz]=size(mask);

ind=find(mask>0);  % all pixels
[indx indy indz]=ind2sub([nx ny nz],ind);
npix=size(ind,1);

if k_non_unif==1
    if sum(abs(size(kmap)-[nx ny nz]))>0
        error('Input mask and thermal conductivity map do not have the same dimensions.');
    end
    dkdx=zeros(nx,ny,nz);
    dkdx(2:end-1,:,:)=( kmap(3:end,:,:)-kmap(1:end-2,:,:) )/DX(1)/2;  % central difference

    dkdy=zeros(nx,ny,nz);
    dkdy(:,2:end-1,:)=( kmap(:,3:end,:)-kmap(:,1:end-2,:) )/DX(2)/2;

    dkdz=zeros(nx,ny,nz);
    dkdz(:,:,2:end-1)=( kmap(:,:,3:end)-kmap(:,:,1:end-2) )/DX(3)/2;
    
    tmp2=zeros(nx,ny,nz);
end

fprintf('Computing Fourier operator ');
sbuf=[];
tmp=zeros(nx,ny,nz);
L=sparse(npix,npix);

idx2=1/DX(1)^2;
idy2=1/DX(2)^2;
idz2=1/DX(3)^2;

ker_mask=zeros(3,3,3);
ker_mask(:,:,1)=[0 0    0; 
                 0 idz2 0;
                 0 0    0];
            
ker_mask(:,:,2)=[0    idy2 0   ;
                 idx2 0    idx2;
                 0    idy2 0   ];
            
ker_mask(:,:,3)=[0 0 0   ; 
                 0 idz2 0;
                 0 0    0];

for i=1:npix
    
    if mod(i,100)==0
        for j=1:size(sbuf,2)
            fprintf('\b');
        end
        sbuf=sprintf('[%d/%d]',i,npix);
        fprintf(sbuf);
    end
    
    if mask(ind(i))==1 || mask(ind(i))==2 || mask(ind(i))==3  % only compute Laplacian operator for pixels within object, i.e. exclude ghost nodes        
        tmp(:)=0;  % reset
        
        kfac=( kmap(indx(i)-1:indx(i)+1,indy(i)-1:indy(i)+1,indz(i)-1:indz(i)+1) + kmap(indx(i),indy(i),indz(i)) ) .* ker_mask;
        kfac(2,2,2)=-sum(sum(sum(kfac)));
        
        % tmp( indx(i)-1:indx(i)+1,indy(i)-1:indy(i)+1,indz(i)-1:indz(i)+1 )=tmp( indx(i)-1:indx(i)+1,indy(i)-1:indy(i)+1,indz(i)-1:indz(i)+1 ) .* kmap( indx(i)-1:indx(i)+1,indy(i)-1:indy(i)+1,indz(i)-1:indz(i)+1 );
        tmp( indx(i)-1:indx(i)+1,indy(i)-1:indy(i)+1,indz(i)-1:indz(i)+1 )=kfac/2;

        % gradient of k-map contribution                                         
%         if k_non_unif==1        
%             tmp2(:)=0;
%             
%             tmp2( indx(i)-1:indx(i)+1,indy(i)          ,indz(i)           ) = dkdx( indx(i)-1:indx(i)+1,indy(i)          ,indz(i)           ) .* reshape([-1;0;1],3,1,1)/DX(1)/2;
%             tmp2( indx(i)          ,indy(i)-1:indy(i)+1,indz(i)           ) = dkdy( indx(i)          ,indy(i)-1:indy(i)+1,indz(i)           ) .* reshape([-1;0;1],1,3,1)/DX(2)/2 + tmp2( indx(i)          ,indy(i)-1:indy(i)+1,indz(i)           );
%             tmp2( indx(i)          ,indy(i)          ,indz(i)-1:indz(i)+1 ) = dkdz( indx(i)          ,indy(i)          ,indz(i)-1:indz(i)+1 ) .* reshape([-1;0;1],1,1,3)/DX(3)/2 + tmp2( indx(i)          ,indy(i)          ,indz(i)-1:indz(i)+1 );
% 
%             L(:,i)=sparse(tmp(ind) + tmp2(ind) );
%         else
%            L(:,i)=sparse(tmp(ind));
%         end

        L(:,i)=sparse(tmp(ind));
        
    end
end

fprintf('\n');




































