


function L=pbhs_create_laplacian_operator(mask,DX,kmap,k_non_unif)
% function L=pbhs_create_laplacian_operator(mask,DX,kmap,k_non_unif)

[nx ny nz]=size(mask);
npix_tot=nx*ny*nz;

ind=find(mask>0);  % all pixels
[indx indy indz]=ind2sub([nx ny nz],ind);
npix=size(ind,1);

if k_non_unif==1    
    if sum(abs(size(kmap)-[nx ny nz]))>0
        error('Input mask and thermal conductivity map do not have the same dimensions.');
    end
    
    % gradient thermal conductivity
    dkdx=zeros(nx,ny,nz);
    dkdy=zeros(nx,ny,nz);
    dkdz=zeros(nx,ny,nz);  
    
    dkdx(1:end-1,:,:)=( kmap(2:end,:,:)-kmap(1:end-1,:,:) )/DX(1);
    dkdy(:,1:end-1,:)=( kmap(:,2:end,:)-kmap(:,1:end-1,:) )/DX(2);
    dkdz(:,:,1:end-1)=( kmap(:,:,2:end)-kmap(:,:,1:end-1) )/DX(3);   
    
    DKDX=sparse(npix_tot,npix_tot);
    DKDY=sparse(npix_tot,npix_tot);
    DKDZ=sparse(npix_tot,npix_tot);
    
    DKDX=setsparse( DKDX,ind,ind,dkdx(ind) );
    DKDY=setsparse( DKDY,ind,ind,dkdy(ind) );
    DKDZ=setsparse( DKDZ,ind,ind,dkdz(ind) );
    
    EYE=sparse(npix_tot,npix_tot);
    EYE=setsparse( EYE,ind,ind,1 );
end

fprintf('Computing Fourier operator ');
L_full=sparse(npix_tot,npix_tot);

sbuf=[];

% -x term (Laplacian)
for j=1:size(sbuf,2)
    fprintf('\b');
end
sbuf=sprintf('[1/7]');
fprintf(sbuf);

a=indx-1; b=indy; c=indz;
tmp=find( a>0 & a<=nx & b>0 & b<=ny & c>0 & c<=nz );
a=a(tmp); b=b(tmp); c=c(tmp);
ind2=sub2ind( [nx ny nz],a,b,c );
ind2=ind2( find(mask(ind2)>0) ); % only retain those indices that are in the mask
[ind2_x ind2_y ind2_z]=ind2sub( [nx ny nz],ind2 );

ind3=sub2ind( [nx ny nz], ind2_x+1 , ind2_y ,ind2_z );
% L_full( sub2ind([npix_tot npix_tot],ind2,ind3) ) = 1/DX(1)^2;
L_full=setsparse( L_full,ind2,ind3,1/DX(1)^2,@plus );  % much faster

if k_non_unif==1
    % -x term (gradient)
    GRADX=sparse(npix_tot,npix_tot);
    GRADX=( setsparse( GRADX,ind2,ind3,-1 ) + EYE )*DKDX/DX(1);
end

% +x term (Laplacian)
for j=1:size(sbuf,2)
    fprintf('\b');
end
sbuf=sprintf('[2/7]');
fprintf(sbuf);

a=indx+1; b=indy; c=indz;
tmp=find( a>0 & a<=nx & b>0 & b<=ny & c>0 & c<=nz );
a=a(tmp); b=b(tmp); c=c(tmp);
ind2=sub2ind( [nx ny nz],a,b,c );
ind2=ind2( find(mask(ind2)>0) ); % only retain those indices that are in the mask
[ind2_x ind2_y ind2_z]=ind2sub( [nx ny nz],ind2 );

ind3=sub2ind( [nx ny nz], ind2_x-1 , ind2_y ,ind2_z );
L_full=setsparse( L_full,ind2,ind3,1/DX(1)^2,@plus );

% -y term (Laplacian)
for j=1:size(sbuf,2)
    fprintf('\b');
end
sbuf=sprintf('[3/7]');
fprintf(sbuf);

a=indx; b=indy-1; c=indz;
tmp=find( a>0 & a<=nx & b>0 & b<=ny & c>0 & c<=nz );
a=a(tmp); b=b(tmp); c=c(tmp);
ind2=sub2ind( [nx ny nz],a,b,c );
ind2=ind2( find(mask(ind2)>0) ); % only retain those indices that are in the mask
[ind2_x ind2_y ind2_z]=ind2sub( [nx ny nz],ind2 );

ind3=sub2ind( [nx ny nz], ind2_x , ind2_y+1 ,ind2_z );
L_full=setsparse( L_full,ind2,ind3,1/DX(2)^2,@plus );

if k_non_unif==1
    % -y term (gradient)
    GRADY=sparse(npix_tot,npix_tot);
    GRADY=( setsparse( GRADY,ind2,ind3,-1 ) + EYE )*DKDY/DX(2);
end

% +y term (Laplacian)
for j=1:size(sbuf,2)
    fprintf('\b');
end
sbuf=sprintf('[4/7]');
fprintf(sbuf);

a=indx; b=indy+1; c=indz;
tmp=find( a>0 & a<=nx & b>0 & b<=ny & c>0 & c<=nz );
a=a(tmp); b=b(tmp); c=c(tmp);
ind2=sub2ind( [nx ny nz],a,b,c );
ind2=ind2( find(mask(ind2)>0) ); % only retain those indices that are in the mask
[ind2_x ind2_y ind2_z]=ind2sub( [nx ny nz],ind2 );

ind3=sub2ind( [nx ny nz], ind2_x , ind2_y-1 ,ind2_z );
L_full=setsparse( L_full,ind2,ind3,1/DX(2)^2,@plus );

% -z term (Laplacian)
for j=1:size(sbuf,2)
    fprintf('\b');
end
sbuf=sprintf('[5/7]');
fprintf(sbuf);

a=indx; b=indy; c=indz-1;
tmp=find( a>0 & a<=nx & b>0 & b<=ny & c>0 & c<=nz );
a=a(tmp); b=b(tmp); c=c(tmp);
ind2=sub2ind( [nx ny nz],a,b,c );
ind2=ind2( find(mask(ind2)>0) ); % only retain those indices that are in the mask
[ind2_x ind2_y ind2_z]=ind2sub( [nx ny nz],ind2 );

ind3=sub2ind( [nx ny nz], ind2_x , ind2_y ,ind2_z+1 );
L_full=setsparse( L_full,ind2,ind3,1/DX(3)^2,@plus );

if k_non_unif==1
    % -z term (gradient)
    GRADZ=sparse(npix_tot,npix_tot);
    GRADZ=( setsparse( GRADZ,ind2,ind3,-1 ) + EYE )*DKDZ/DX(3);
end

% +z term (Laplacian)
for j=1:size(sbuf,2)
    fprintf('\b');
end
sbuf=sprintf('[6/7]');
fprintf(sbuf);

a=indx; b=indy; c=indz+1;
tmp=find( a>0 & a<=nx & b>0 & b<=ny & c>0 & c<=nz );
a=a(tmp); b=b(tmp); c=c(tmp);
ind2=sub2ind( [nx ny nz],a,b,c );
ind2=ind2( find(mask(ind2)>0) ); % only retain those indices that are in the mask
[ind2_x ind2_y ind2_z]=ind2sub( [nx ny nz],ind2 );

ind3=sub2ind( [nx ny nz], ind2_x , ind2_y ,ind2_z-1 );
L_full=setsparse( L_full,ind2,ind3,1/DX(3)^2,@plus );

% reduce size to non-zero pixels and add diagonal term
for j=1:size(sbuf,2)
    fprintf('\b');
end
sbuf=sprintf('[7/7]');
fprintf(sbuf);

L_full=setsparse( L_full,ind,ind,-2*( 1/DX(1)^2 + 1/DX(2)^2 + 1/DX(3)^2 ),@plus );

KDIAG=sparse(npix,npix);
KDIAG=setsparse( KDIAG,[1:npix]',[1:npix]',kmap(ind) );

L=sparse(npix,npix);
L=L_full(ind,ind)*KDIAG;

if k_non_unif==1
    L=L + GRADX(ind,ind) + GRADY(ind,ind) + GRADZ(ind,ind);
end

fprintf('\n');















